// OTMaxTest.cpp : Defines the entry point for the console application.
//


#include "stdafx.h"

#include "..\OTMax.h"
#include <comdef.h>

#include "../Utils/StrUtils.h"
#include "TranString.h"

#define USEPTR(X) _COM_SMARTPTR_TYPEDEF(X,__uuidof(X))

USEPTR(IOpenTool);
USEPTR(IOTDescribe);
USEPTR(IOTChain);
USEPTR(IEnumUnknown);
USEPTR(IOTConnector);
USEPTR(IOTConnectorDescribe);
USEPTR(IOTOutput);

USEPTR(IMaxOTConnectPoints);
USEPTR(IMaxOTConnectVertices);
USEPTR(IMaxOTConnectTopography);
USEPTR(IOTDataIsPointList);
USEPTR(IOTDataIsPointCloud);
USEPTR(IOTDataIsTopologyList);
USEPTR(IOTDataList);
USEPTR(IOTDataIsPoint);
USEPTR(IOTDataIsPointSet);
USEPTR(IOTDataIsTopology);
USEPTR(IOTDataVector);
USEPTR(IOTDataIndexSet);
USEPTR(IOTDataIndexList);


#include <iostream>

using namespace std;

#define TEST(X) do { if (!bool(X)) { cout<<"FAIL "<< #X<<endl; break; } } while (0)
#define COMTEST(X) { HRESULT h=(X); if (FAILED(h)) { cout<<"FAIL "<< #X <<":"<<hex<<(h)<<endl; break; } }



int main(int p_nArgs,char *p_asArgs[])
{
	HRESULT t_hInit=CoInitialize(0);
	if (FAILED(t_hInit))
		return t_hInit;

	do
	{

		// Get a pointer to the main tool
		IOpenToolPtr t_spIOTMaxScene(__uuidof(MaxScene));		TEST(t_spIOTMaxScene);

		// Get some description info from the main tool
		IOTDescribePtr t_spIOTDescribe(t_spIOTMaxScene);			TEST(t_spIOTDescribe);

		// Get its name
		wchar_t* t_pwsX;
		wchar_t* t_pwsY;
		COMTEST(t_spIOTDescribe->Name(&t_pwsX,&t_pwsY));
		cout<< (const char*)TranString(t_pwsX) <<" [Aliases: "<<(const char*)TranString(t_pwsY)<<"]" <<endl;
		StrFree(&t_pwsX);
		StrFree(&t_pwsY);

		// Get the tool's chaining interface
		IOTChainPtr t_spIOTChain(t_spIOTMaxScene);					TEST(t_spIOTChain);

		IEnumUnknownPtr t_spIEnumUnknown(t_spIOTMaxScene);					TEST(t_spIEnumUnknown);

		IUnknownPtr t_spIUnknown;
		HRESULT t_h;
		for (t_spIEnumUnknown->Reset(); (t_h=t_spIEnumUnknown->Next(1u,&t_spIUnknown,0))==S_OK; )
		{	IOTConnectorPtr t_spIOTConnector;
			if (t_spIUnknown->QueryInterface(&t_spIOTConnector)==S_OK)	// Do we have a connector?
			{	IMaxOTConnectPointsPtr t_spIMaxOTConnectPoints;
				IMaxOTConnectVerticesPtr t_spIMaxOTConnectVertices;
				IMaxOTConnectTopographyPtr t_spIMaxOTConnectTopography;

				if (t_spIUnknown->QueryInterface(&t_spIMaxOTConnectPoints)==S_OK)	// Is it the points connector?
				{	cout<<"Points"<<endl;

					IOTDataIsPointListPtr t_spIOTDataIsPointList(t_spIMaxOTConnectPoints);
					if (t_spIOTDataIsPointList) // Is this points data?
					{	IOTDataListPtr t_spIOTDataList(t_spIOTDataIsPointList);
						if (t_spIOTDataList)	// Got Point data in a shape we understand?
						{	IOTDataIsPointPtr t_spIOTDataIsPoint;
							otIndex t_iStart=0;
							otCardinal t_uReturned=0;
							otIndex t_iRemain=0;

							do
							{	cout<<"Point "<<(unsigned)t_iStart<<"=[";
								
								HRESULT t_h=t_spIOTDataList->List(__uuidof(IOTDataIsPoint),(IUnknown**)&t_spIOTDataIsPoint,1u,&t_iStart,&t_uReturned,&t_iRemain);
								COMTEST(t_h);
								
								IOTDataVectorPtr t_spIOTDataVector(t_spIOTDataIsPoint);
								if (t_spIOTDataVector)
								{	otReal t_rOrdinate;
									otIndex t_iStart=0;
									otCardinal t_uReturned=0;
									otIndex t_iRemain=0;

									do
									{	if (t_iStart)
											cout<<",";

										HRESULT t_h=t_spIOTDataVector->Vector(&t_rOrdinate,1u,&t_iStart,&t_uReturned,&t_iRemain);
										COMTEST(t_h);
										
										cout<<t_rOrdinate;

									} while (t_iRemain);
									cout<<"]"<<endl;
								}
								
							} while (t_iRemain);
							
						}

					}
				}
				else
					if (t_spIUnknown->QueryInterface(&t_spIMaxOTConnectVertices)==S_OK)	// Is it vertices?
					{	cout<<"Vertices"<<endl;

						IOTDataIsPointCloudPtr t_spIOTDataIsPointCloud(t_spIMaxOTConnectVertices);
						if (t_spIOTDataIsPointCloud) // Is this a point cloud?
						{	IOTDataListPtr t_spIOTDataList(t_spIOTDataIsPointCloud);
							if (t_spIOTDataList)	// Got Point data in a shape we understand?
							{	IOTDataIsPointSetPtr t_spIOTDataIsPointSet;
								otIndex t_iStart=0;
								otCardinal t_uReturned=0;
								otIndex t_iRemain=0;

								do
								{	cout<<"Vertex "<<(unsigned)t_iStart<<"=["<<hex;
									
									HRESULT t_h=t_spIOTDataList->List(__uuidof(IOTDataIsPointSet),(IUnknown**)&t_spIOTDataIsPointSet,1u,&t_iStart,&t_uReturned,&t_iRemain);
									COMTEST(t_h);
									
									IOTDataIndexSetPtr t_spIOTDataIndexSet(t_spIOTDataIsPointSet);
									if (t_spIOTDataIndexSet)
									{	otIndex t_iBuffer;
										otIndex t_iStart=0;
										otCardinal t_uReturned=0;
										otIndex t_iRemain=0;

										do
										{	if (t_iStart)
												cout<<",";

											HRESULT t_h=t_spIOTDataIndexSet->IndexSet(&t_iBuffer,1u,&t_iStart,&t_uReturned,&t_iRemain);
											COMTEST(t_h);
											
											cout<<(unsigned)(t_iBuffer>>32)<<":"<<(unsigned)t_iBuffer;

										} while (t_iStart);
										cout<<"]"<<endl;
//										cout<<"Start="<<(unsigned)t_iStart<<", Returned="<<t_uReturned<<", Remain="<<(unsigned)t_iRemain<<endl;
									}
									
								} while (t_iRemain);
								
							}

						}
					}
					else
						if (t_spIUnknown->QueryInterface(&t_spIMaxOTConnectTopography)==S_OK)	// Is it Topography?
						{	cout<<"Topography"<<endl;

							IOTDataIsTopologyListPtr t_spIOTDataIsTopologyList(t_spIMaxOTConnectTopography);
							if (t_spIOTDataIsTopologyList) // Is this a topology list?
							{	IOTDataListPtr t_spIOTDataList(t_spIOTDataIsTopologyList);
								if (t_spIOTDataList)	// Got Point data in a shape we understand?
								{	IOTDataIsTopologyPtr t_spIOTDataIsTopology;
									otIndex t_iStart=0;
									otCardinal t_uReturned=0;
									otIndex t_iRemain=0;

									do
									{	cout<<"Face "<<(unsigned)t_iStart<<"=["<<hex;
										
										HRESULT t_h=t_spIOTDataList->List(__uuidof(IOTDataIsTopology),(IUnknown**)&t_spIOTDataIsTopology,1u,&t_iStart,&t_uReturned,&t_iRemain);
										COMTEST(t_h);
										
										IOTDataIndexListPtr t_spIOTDataIndexList(t_spIOTDataIsTopology);
										if (t_spIOTDataIndexList)
										{	otIndex t_iBuffer;
											otIndex t_iStart=0;
											otCardinal t_uReturned=0;
											otIndex t_iRemain=0;

											do
											{	if (t_iStart)
													cout<<",";

												HRESULT t_h=t_spIOTDataIndexList->IndexList(&t_iBuffer,1u,&t_iStart,&t_uReturned,&t_iRemain);
												COMTEST(t_h);
												
												cout<<(unsigned)(t_iBuffer>>32)<<":"<<(unsigned)t_iBuffer;

											} while (t_iRemain);
											cout<<"]"<<endl;
	//										cout<<"Start="<<(unsigned)t_iStart<<", Returned="<<t_uReturned<<", Remain="<<(unsigned)t_iRemain<<endl;
										}
										
									} while (t_iRemain);
									
								}

							}


						}
			}
		}
		COMTEST(t_h);






// END OF TEST
	
		cout<<"OK"<<endl;
	}
	while (0);

	CoUninitialize();
	cout<<"\nEND."<<endl;
	return 0;
}
