#if !defined( H_TranString_H )
#define H_TranString_H

#include <string.h>

class TranString
{
private:
	mutable wchar_t* m_ws;
	mutable char* m_s;
protected:
	const wchar_t* Wide() const;	// Ensure the wide version is available
	const char* Narrow() const;		// Ensure the narrow version is available
	static wchar_t* Copy(const wchar_t* ws) { return ws?wcscpy(new wchar_t[wcslen(ws)+1],ws):0; }
	static char* Copy(const char* s) { return s?strcpy(new char[strlen(s)+1],s):0; }
public:
    TranString():m_ws(0),m_s(0) { }
    TranString(const wchar_t* ws):m_ws(Copy(ws)),m_s(0) { }
    TranString(const char* s):m_ws(0),m_s(Copy(s)) { }
    TranString(const TranString& t):m_ws(Copy(t.m_ws)),m_s(Copy(t.m_s)) { }
	~TranString() { delete m_ws; delete m_s; }
	operator const char*() const { return Narrow(); }
	operator const wchar_t*() const { return Wide(); }
};

inline const char* TranString::Narrow() const
{	if (m_s)
		return m_s;	// Already known

	if (!m_ws)
		return "";	// Nothing to translate

	int t_nLength
	=	WideCharToMultiByte
		(	CP_THREAD_ACP
		,	WC_COMPOSITECHECK
		,	m_ws
		,	-1
		,	0
		,	0
		,	"_"
		,	0
		)
	;

	if (t_nLength<=0)
		return "";

	m_s=new char[t_nLength+1];

	int t_nWritten
	=	WideCharToMultiByte
		(	CP_THREAD_ACP
		,	WC_COMPOSITECHECK
		,	m_ws
		,	-1
		,	m_s
		,	t_nLength+1
		,	"_"
		,	0
		)
	;

	return m_s;	// Now we have a narrow version
}

inline const wchar_t* TranString::Wide() const
{	if (m_ws)
		return m_ws;	// Already known

	if (!m_s)
		return L"";	// Nothing to translate

	int t_nLength
	=	MultiByteToWideChar
		(	CP_THREAD_ACP
		,	MB_PRECOMPOSED
		,	m_s
		,	-1
		,	0
		,	0
		);

	if (t_nLength<=0)
		return L"";

	m_ws=new wchar_t[t_nLength+1];

	int t_nWritten
	=	MultiByteToWideChar
		(	CP_THREAD_ACP
		,	MB_PRECOMPOSED
		,	m_s
		,	-1
		,	m_ws
		,	t_nLength+1
		);

	return m_ws;	// Now we have a wide version
}

#endif
